package com.tencent.flutter_integration;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.HashMap;

import io.flutter.embedding.android.FlutterFragment;
import io.flutter.embedding.android.FlutterView;
import io.flutter.embedding.engine.FlutterEngine;
import io.flutter.embedding.engine.dart.DartExecutor;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.embedding.engine.systemchannels.LifecycleChannel;

public class FlutterIntegrationFragment extends FlutterFragment {

    public String routeName;
    public Object params;
    private FlutterIntegrationEngineContainer mContainer;
    private String mPageId;
    private FlutterView mFlutterView;

    public FlutterIntegrationFragment() {
        super();
        mContainer = FlutterIntegrationManager.sharedInstance().container;
    }

    public static FlutterIntegrationFragment newInstance(String routeName) {
        return newInstance(routeName, null);
    }

    public static FlutterIntegrationFragment newInstance(String routeName, Object params) {
        FlutterIntegrationFragment fragment =
                new NewEngineFragmentBuilder(FlutterIntegrationFragment.class).initialRoute("/").transparencyMode(FlutterView.TransparencyMode.opaque)
                        .build();
        fragment.routeName = routeName;
        fragment.params = params;

        return fragment;
    }

    @Override
    public FlutterEngine provideFlutterEngine(Context context) {
        return mContainer.flutterEngine;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        View splashView = super.onCreateView(inflater, container, savedInstanceState);
        if ((splashView instanceof FrameLayout)) {
            mFlutterView = (FlutterView) ((FrameLayout) splashView).getChildAt(0);
        }
        return splashView;
    }

    @Override
    public void onResume() {
        super.onResume();

        mContainer.fragment = this;
        mFlutterView.attachToFlutterEngine(mContainer.flutterEngine);

        mContainer.flutterEngine.getActivityControlSurface().attachToActivity(
                getActivity(),
                getLifecycle()
        );
        openPage();
        Log.e("FlutterIntegration", String.format("onResume, pageId:%s", mPageId));
    }

    @Override
    public void onPause() {
        super.onPause();

        Log.e("FlutterIntegration", String.format("onPause, pageId:%s", mPageId));
    }

    @Override
    public void onStop() {
        super.onStop();
//        mContainer.flutterEngine.getLifecycleChannel().appIsResumed();

        if (mContainer.fragment == this) {
            mContainer.fragment = null;
        } else if (mContainer.fragment != null) {
            mContainer.fragment.onResume();
        }
    }

    @Override
    public void onDetach() {
        super.onDetach();
//        mContainer.flutterEngine.getLifecycleChannel().appIsResumed();

        if (mContainer.fragment == this) {
            mContainer.fragment = null;
        } else if (mContainer.fragment != null) {
            mContainer.fragment.onResume();
        }
        Log.e("FlutterIntegration", String.format("onDetach, pageId:%s", mPageId));
    }

    @Override
    public void onDestroyView() {
        super.onDestroyView();
        removePage();
        Log.e("FlutterIntegration", String.format("onDestroyView, pageId:%s", mPageId));
    }

    @Override
    public boolean shouldDestroyEngineWithHost() {
        return false;
    }

    @Override
    public boolean shouldAttachEngineToActivity() {
        return false;
    }

    public void detachEngine() {
        mFlutterView.detachFromFlutterEngine();
    }

    @SuppressLint("DefaultLocale")
    private String getPageId() {
        if (mPageId == null || mPageId.length() <= 0) {
            mPageId = String.format("%s_%d", routeName, System.currentTimeMillis());
        }
        return mPageId;
    }

    private void openPage() {
        HashMap<String, Object> map = new HashMap<>();
        map.put("routeName", routeName);
        map.put("pageId", getPageId());
        if (params != null) {
            map.put("params", params);
        }

        mContainer.methodChannel.invokeMethod("openPage", map);
    }

    private void removePage() {
        HashMap<String, Object> map = new HashMap<>();
        map.put("pageId", mPageId);

        mContainer.methodChannel.invokeMethod("removePage", map);
    }
}
